@@ -67,7 +67,7 @@ class relation (object):
         with spaces, you should avoid using it.'''
         
         fp=file(filename,'w') #Opening file in write mode
-        if comma_separated:
+        if comma_separated: #writing csv
             writer=csv.writer(fp) #Creating csv writer
             
             #It wants an iterable containing iterables
@@ -77,7 +77,7 @@ class relation (object):
             
             #Writing content, already in the correct format
             writer.writerows(self.content)
-        else:
+        else: #Writing in the old, deprecated, format
             res=""
             res+=" ".join(self.header.attributes)
             
@@ -143,7 +143,7 @@ class relation (object):
         Will delete duplicate items
         If an empty list or no parameters are provided, returns None'''    
         #Parameters are supplied in a list, instead with multiple parameters
-        if attributes[0].__class__ == list().__class__:
+        if isinstance(attributes[0],list):
             attributes=attributes[0]
         
         #Avoiding duplicated attributes
@@ -153,6 +153,10 @@ class relation (object):
                 attributes1.append(i)
         attributes=attributes1
         
+        #If source and dest has the same number of attributes, we are just rearranging
+        #so we won't need to check for duplicated entries
+        attributes_same_count=len(attributes)==len(self.header.attributes)
+        
         ids=self.header.getAttributesId(attributes)
         
         if len(ids)==0:
@@ -169,11 +173,9 @@ class relation (object):
             row=[]
             for j in ids:
                 row.append(i[j])
-            if row not in newt.content:#Avoids duplicated items
+            if attributes_same_count or row not in newt.content:
                 newt.content.append(row)
         return newt
-        
-        
     
     def rename(self,params):
         '''Operation rename. Takes a dictionary
@@ -446,8 +448,14 @@ class relation (object):
                     attributes[self.header.attributes[j]]=i[j]
             if eval(expr,attributes): #If expr is true, changing the tuple
                 affected+=1
+                new_tuple=list(i)
+                #Deleting the tuple, instead of changing it, so other
+                #relations can still point to the same list without
+                #being affected.
+                self.content.remove(i) 
                 for k in range(len(keys)):
-                    i[f_ids[k]]=str(dic[keys[k]])
+                    new_tuple[f_ids[k]]=str(dic[keys[k]])
+                self.content.append(new_tuple)
         return affected
     def insert(self,values):
         '''Inserts a tuple in the relation.
