@@ -20,20 +20,30 @@
 # author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>
 
 import sys
+import sip
 from PyQt4 import QtCore, QtGui
 from relational_gui import maingui, about
 from relational import relation, parser
 
-version="0.9"
+version="0.10"
 about.version=version
 
 if __name__ == "__main__":
     if len (sys.argv) > 1 and sys.argv[1] == "-v":
         print version
         sys.exit(0)
+        
+    try:
+        import psyco
+        psyco.full()
+    except:
+        pass
+    
     app = QtGui.QApplication(sys.argv)
     Form = QtGui.QWidget()
     
+    Form.setFont(QtGui.QFont("Dejavu Sans Bold"))
+    
     ui = maingui.Ui_Form()
     ui.setupUi(Form)
     Form.show()
