@@ -17,7 +17,7 @@
 # 
 # author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>
 from PyQt4 import QtCore, QtGui
-from relational import relation, parser
+from relational import relation, parser, optimizer
 
 import sys
 import about
@@ -30,8 +30,37 @@ class Ui_Form(object):
         self.About=None
         self.Survey=None
         self.relations={} #Dictionary for relations
+        self.undo=[] #UndoQueue for relations
         self.selectedRelation=None
+    def load_query(self,*index):
+        self.txtQuery.setText(self.savedQ.itemData(index[0]).toString())
         
+    def save_query(self):
+        defname=""
+        res=QtGui.QInputDialog.getText(self.Form, QtGui.QApplication.translate("Form", "New query"),QtGui.QApplication.translate("Form", "Insert the name for the query"),
+        QtGui.QLineEdit.Normal,defname)
+        if res[1]==False:
+            return
+            
+        self.savedQ.addItem(res[0],QtCore.QVariant(self.txtQuery.text()))
+    def toggle_advanced(self):
+        if self.advancedBox.isVisible():
+            self.advancedBox.hide()
+        else:
+            self.advancedBox.show()
+        
+    def undo_optimize(self):
+        try:
+            self.txtQuery.setText(self.undo.pop())
+        except:#Nothing to restore
+            pass
+    def optimize(self):
+        self.undo.insert(len(self.undo),self.txtQuery.text()) #Storing the query in undo list
+        
+        result=optimizer.optimize_all(str(self.txtQuery.text().toUtf8()),self.relations)
+        self.txtQuery.setText(QtCore.QString.fromUtf8(result))
+        
+        #self.txtQuery.setText(result)
     def execute(self):
         try:
             #Converting string to utf8 and then from qstring to normal string
@@ -40,6 +69,8 @@ class Ui_Form(object):
             print query,"-->" , expr #Printing debug
             result=eval(expr,self.relations) #Evaluating the expression
             
+            #self.undo.insert(len(self.undo),self.txtQuery.text()) #Storing the query in undo list
+            
             res_rel=str(self.txtResult.text())#result relation's name
             self.txtResult.setText("") #Sets the result relation name to none
             if len(res_rel)==0: #If no name is set use the default last_
@@ -125,7 +156,7 @@ class Ui_Form(object):
         filename = QtGui.QFileDialog.getOpenFileName(None,QtGui.QApplication.translate("Form", "Load Relation"),"",QtGui.QApplication.translate("Form", "Relations (*.csv);;Old Relations (*.tlb);;Text Files (*.txt);;All Files (*)"))
         
         #Default relation's name
-        f=str(filename.toUtf8()).split(os.sep) #Split the full path
+        f=str(filename.toUtf8()).split('/') #Split the full path
         defname=f[len(f)-1].lower() #Takes only the lowercase filename
         if len(defname)==0:
             return
@@ -352,28 +383,74 @@ class Ui_Form(object):
         self.verticalLayout_3.addWidget(self.groupBox_3)
         self.horizontalLayout_4.addLayout(self.verticalLayout_3)
         self.verticalLayout_7.addLayout(self.horizontalLayout_4)
-        self.horizontalLayout = QtGui.QHBoxLayout()
-        self.horizontalLayout.setObjectName("horizontalLayout")
+        
+        
+        
+        
+        self.queryLayout = QtGui.QHBoxLayout()
+        self.queryLayout.setObjectName("queryLayout")
         self.label = QtGui.QLabel(Form)
         self.label.setObjectName("label")
-        self.horizontalLayout.addWidget(self.label)
+        self.queryLayout.addWidget(self.label)
         self.txtResult = QtGui.QLineEdit(Form)
         self.txtResult.setMaximumSize(QtCore.QSize(70,16777215))
         self.txtResult.setObjectName("txtResult")
-        self.horizontalLayout.addWidget(self.txtResult)
+        self.queryLayout.addWidget(self.txtResult)
         self.label_2 = QtGui.QLabel(Form)
         self.label_2.setObjectName("label_2")
-        self.horizontalLayout.addWidget(self.label_2)
+        self.queryLayout.addWidget(self.label_2)
         self.txtQuery = QtGui.QLineEdit(Form)
         self.txtQuery.setObjectName("txtQuery")
-        self.horizontalLayout.addWidget(self.txtQuery)
+        self.queryLayout.addWidget(self.txtQuery)
         self.cmdExecute = QtGui.QPushButton(Form)
         self.cmdExecute.setAutoDefault(False)
         self.cmdExecute.setDefault(True)
         self.cmdExecute.setFlat(False)
         self.cmdExecute.setObjectName("cmdExecute")
-        self.horizontalLayout.addWidget(self.cmdExecute)
-        self.verticalLayout_7.addLayout(self.horizontalLayout)
+        self.queryLayout.addWidget(self.cmdExecute)
+        
+        self.cmdAdvanced = QtGui.QPushButton(Form)
+        self.cmdAdvanced.setAutoDefault(False)
+        self.cmdAdvanced.setFlat(False)
+        self.cmdAdvanced.setObjectName("cmdAdvanced")
+        self.queryLayout.addWidget(self.cmdAdvanced)
+        
+        
+        self.advancedBox = QtGui.QGroupBox(Form)
+        self.advancedBox.setObjectName("advancedBox")
+        self.advancedLayout = QtGui.QHBoxLayout(self.advancedBox)
+        self.advancedLayout.setObjectName("AdvancedLayout")
+        self.advancedBox.hide()
+        
+        self.cmdOptimize = QtGui.QPushButton(Form)
+        self.cmdOptimize.setAutoDefault(False)
+        self.cmdOptimize.setFlat(False)
+        self.cmdOptimize.setObjectName("cmdOptimize")
+        self.advancedLayout.addWidget(self.cmdOptimize)
+        
+        self.cmdUndoOptimize = QtGui.QPushButton(Form)
+        self.cmdUndoOptimize.setAutoDefault(False)
+        self.cmdUndoOptimize.setDefault(True)
+        self.cmdUndoOptimize.setFlat(False)
+        self.cmdUndoOptimize.setObjectName("cmdUndoOptimize")
+        self.advancedLayout.addWidget(self.cmdUndoOptimize)
+        
+        
+        self.savedQ= QtGui.QComboBox(Form)
+        self.savedQ.addItem(QtGui.QApplication.translate("Form", "Select query"),QtCore.QVariant(None))
+        self.advancedLayout.addWidget(self.savedQ)
+        
+        self.cmdSaveQ = QtGui.QPushButton(Form)
+        self.cmdSaveQ.setAutoDefault(False)
+        self.cmdSaveQ.setFlat(False)
+        self.cmdSaveQ.setObjectName("cmdSaveQ")
+        self.advancedLayout.addWidget(self.cmdSaveQ)
+        
+        
+        
+        self.verticalLayout_7.addLayout(self.queryLayout)
+        self.verticalLayout_7.addWidget(self.advancedBox)
+        
         self.label.setBuddy(self.txtResult)
         self.label_2.setBuddy(self.txtQuery)
 
@@ -394,6 +471,10 @@ class Ui_Form(object):
         QtCore.QObject.connect(self.cmdRename,QtCore.SIGNAL("clicked()"),self.addRename)
         QtCore.QObject.connect(self.cmdArrow,QtCore.SIGNAL("clicked()"),self.addArrow)
         QtCore.QObject.connect(self.cmdExecute,QtCore.SIGNAL("clicked()"),self.execute)
+        QtCore.QObject.connect(self.cmdAdvanced,QtCore.SIGNAL("clicked()"),self.toggle_advanced)
+        QtCore.QObject.connect(self.cmdOptimize,QtCore.SIGNAL("clicked()"),self.optimize)
+        QtCore.QObject.connect(self.cmdUndoOptimize,QtCore.SIGNAL("clicked()"),self.undo_optimize)
+        QtCore.QObject.connect(self.cmdSaveQ,QtCore.SIGNAL("clicked()"),self.save_query)
         QtCore.QObject.connect(self.cmdLoad,QtCore.SIGNAL("clicked()"),self.loadRelation)
         QtCore.QObject.connect(self.cmdSave,QtCore.SIGNAL("clicked()"),self.saveRelation)
         QtCore.QObject.connect(self.cmdUnload,QtCore.SIGNAL("clicked()"),self.unloadRelation)
@@ -401,10 +482,16 @@ class Ui_Form(object):
         QtCore.QObject.connect(self.cmdDelete,QtCore.SIGNAL("clicked()"),self.deleteTuple)
         QtCore.QObject.connect(self.lstRelations,QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem*)"),self.printRelation)
         QtCore.QObject.connect(self.lstRelations,QtCore.SIGNAL("itemClicked(QListWidgetItem*)"),self.showAttributes)
+        QtCore.QObject.connect(self.savedQ,QtCore.SIGNAL("currentIndexChanged(int)"),self.load_query)
+        
         QtCore.QMetaObject.connectSlotsByName(Form)
         Form.setTabOrder(self.txtResult,self.txtQuery)
         Form.setTabOrder(self.txtQuery,self.cmdExecute)
-        Form.setTabOrder(self.cmdExecute,self.lstRelations)
+        Form.setTabOrder(self.txtQuery,self.cmdAdvanced)
+        Form.setTabOrder(self.txtQuery,self.cmdOptimize)
+        Form.setTabOrder(self.txtQuery,self.cmdUndoOptimize)
+        Form.setTabOrder(self.txtQuery,self.savedQ)
+        Form.setTabOrder(self.txtQuery,self.cmdSaveQ)
         Form.setTabOrder(self.lstRelations,self.cmdLoad)
         Form.setTabOrder(self.cmdLoad,self.cmdUnload)
         Form.setTabOrder(self.cmdLoad,self.cmdSave)
@@ -425,6 +512,7 @@ class Ui_Form(object):
         self.cmdAbout.setText(QtGui.QApplication.translate("Form", "Docs", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdSurvey.setText(QtGui.QApplication.translate("Form", "Survey", None, QtGui.QApplication.UnicodeUTF8))
         self.groupBox.setTitle(QtGui.QApplication.translate("Form", "Operators", None, QtGui.QApplication.UnicodeUTF8))
+        self.advancedBox.setTitle(QtGui.QApplication.translate("Form", "Advanced", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdProduct.setToolTip(QtGui.QApplication.translate("Form", "Product operator", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdProduct.setText(QtGui.QApplication.translate("Form", "*", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdDifference.setToolTip(QtGui.QApplication.translate("Form", "Difference operator", None, QtGui.QApplication.UnicodeUTF8))
@@ -464,7 +552,11 @@ class Ui_Form(object):
         self.lstAttributes.setToolTip(QtGui.QApplication.translate("Form", "Shows the attributes of the current relation", None, QtGui.QApplication.UnicodeUTF8))
         self.label.setText(QtGui.QApplication.translate("Form", "Query", None, QtGui.QApplication.UnicodeUTF8))
         self.label_2.setText(QtGui.QApplication.translate("Form", "=", None, QtGui.QApplication.UnicodeUTF8))
-        self.cmdExecute.setText(QtGui.QApplication.translate("Form", "Execute", None, QtGui.QApplication.UnicodeUTF8))    
+        self.cmdExecute.setText(QtGui.QApplication.translate("Form", "Execute", None, QtGui.QApplication.UnicodeUTF8))
+        self.cmdAdvanced.setText(QtGui.QApplication.translate("Form", "Advanced", None, QtGui.QApplication.UnicodeUTF8))
+        self.cmdOptimize.setText(QtGui.QApplication.translate("Form", "Optimize", None, QtGui.QApplication.UnicodeUTF8))
+        self.cmdUndoOptimize.setText(QtGui.QApplication.translate("Form", "Undo optimize", None, QtGui.QApplication.UnicodeUTF8))
+        self.cmdSaveQ.setText(QtGui.QApplication.translate("Form", "Save query", None, QtGui.QApplication.UnicodeUTF8))
 
 if __name__ == "__main__":
     import sys
